@@ -662,27 +662,39 @@ def generate_random_correl_matrix(size: int, corr_max: float):
     return res
 
 
-def find_orthogonal_vector(v):
+def find_orthogonal_vector(v: object) -> object:
     if type(v) is list:
         v = pd.DataFrame(v)
     n, _ = v.shape
     assert (n > 1)
     assert (not all([v.iloc[i, 0] == 0 for i in range(n)]))
-    istar = np.argmin([1 if v.iloc[i, 0] != 0 else 0 for i in range(n)])
-    w = pd.DataFrame(index=range(n), columns=["C"])
-    w.fillna(value=0, inplace=True)
-    istarplus1 = (istar + 1) % n
-    w.iloc[istar, 0] = -v.iloc[istarplus1, 0]
-    w.iloc[istarplus1, 0] = v.iloc[istar, 0]
+    w = pd.DataFrame(np.zeros(n))
+    if n == 2:
+        w.iloc[0, 0] = v.iloc[1, 0]
+        w.iloc[1, 0] = -v.iloc[0, 0]
+    else:
+        istar = np.argmin([1 if v.iloc[i, 0] != 0 else 0 for i in range(n)])
+        w.iloc[:, 0] = np.random.uniform(-1, 1, n)
+        vw = np.dot(v.T, w)[0, 0]
+        w.iloc[istar, 0] = - vw / v.iloc[istar, 0]
     return w
 
 
 def generate_vectors_bouquet(vpivot, bouquet_size: int, min_corr: float) -> pd.DataFrame:
-
-    pass
+    vpivot = pd.DataFrame(vpivot)
+    df_bouquet = pd.DataFrame(index=vpivot.index, columns=range(bouquet_size))
+    epsilon = (1 - min_corr)**0.5
+    for icol in range(bouquet_size):
+        w = find_orthogonal_vector(vpivot)
+        df_bouquet.iloc[:, icol] = vpivot + epsilon * w
+    return df_bouquet
 
 
-def generate_clusterized_vectors(nclusters: int, nbvect: int, max_inter_corr: float, min_intra_corr: float):
+def generate_clusterized_vectors(veclen: int,
+                                 nclusters: int,
+                                 nbvect: int,
+                                 max_inter_corr: float,
+                                 min_intra_corr: float):
 
     pass
 
